@@ -1,5 +1,7 @@
-import typing as T
+import mimetypes
+import os
 import requests
+import typing as T
 from uuid import uuid4
 
 from pycollimator.i18n import N
@@ -39,7 +41,7 @@ class Api:
         try:
             url = GlobalVariables.url() + api
             headers = headers or GlobalVariables.custom_headers()
-            headers["X-Collimator-API-Caller"] = "python-notebook"
+            headers["X-Collimator-API-Caller"] = "pycollimator"
             headers["X-Request-ID"] = str(uuid4())
             if body is None:
                 Log.trace(method, url)
@@ -53,6 +55,7 @@ class Api:
                 response = requests.request(method, url, files=body, headers=headers)
             cls._handle_error(response, headers)
             Log.trace(method, "response:", response.text)
+            Log.trace(method, "response:", response.status_code)
             if response_type == "text":
                 return response.text
             return response.json()
@@ -89,6 +92,14 @@ class Api:
         api = f"{Api.BASE_PATH}/projects/{project_uuid}"
         return cls.get(api)
 
+    @classmethod
+    def get_project_by_name(cls, name: str) -> dict:
+        projects = cls.get_projects()["projects"]
+        for p in projects:
+            if p["title"] == name:
+                return p
+        return {}
+
     @classmethod
     def get_model(cls, model_uuid: str) -> dict:
         api = f"{Api.BASE_PATH}/models/{model_uuid}"
@@ -159,7 +170,41 @@ class Api:
     #     api = f"{Api.BASE_PATH}/models/{model_uuid}/configuration"
     #     return cls.put(api, body)
 
+    @classmethod
+    def model_parameter_update(cls, model_uuid: str, body: dict) -> dict:
+        api = f"{Api.BASE_PATH}/models/{model_uuid}/parameters"
+        return cls.put(api, body)
+
     @classmethod
     def linearization_results_csv(cls, model_uuid: str, simulation_uuid: str) -> str:
         api = f"{Api.BASE_PATH}/models/{model_uuid}/simulations/{simulation_uuid}/lin_results"
         return cls._call_api(api, method="GET", response_type="text")
+
+    @classmethod
+    def upload_file(cls, filepath: str, project_uuid: str = None, overwrite: bool = False) -> str:
+        project_uuid = project_uuid or GlobalVariables.project_uuid()
+        content_type, _ = mimetypes.guess_type(filepath)
+        size = os.stat(filepath).st_size
+
+        response = cls._call_api(
+            f"{Api.BASE_PATH}/projects/{project_uuid}/files",
+            method="POST",
+            body={
+                "name": os.path.basename(filepath),
+                "content_type": content_type or "application/octet-stream",
+                "size": size,
+                "overwrite": overwrite,
+            },
+        )
+
+        file_uuid = response["summary"]["uuid"]
+        s3_url = response["put_presigned_url"]
+
+        with open(filepath, "rb") as f:
+            response = requests.put(s3_url, data=f, headers={"Content-Type": content_type, "Content-Length": str(size)})
+
+        if response.status_code != 200:
+            Log.debug("API error, status_code:", response.status_code, "response:", response.json)
+            raise CollimatorApiError(f"Failed to upload to S3. Detailed error:\n{response.text}")
+
+        return cls._call_api(f"{Api.BASE_PATH}/projects/{project_uuid}/files/{file_uuid}/process", method="POST")
