@@ -16,7 +16,7 @@ from pycollimator.i18n import N
 from pycollimator.log import Log
 from pycollimator.projects import get_project
 from pycollimator.simulation_hashed_file import SimulationHashedFile
-from pycollimator.utils import is_uuid
+from pycollimator.utils import is_path, is_uuid
 
 # from collimator.simulations import run_simulation
 
@@ -210,7 +210,10 @@ class ParameterSet:
         return self._parameters.__repr__()
 
     def __getitem__(self, key):
-        return self._parameters[key]
+        try:
+            return self._parameters[key]
+        except KeyError:
+            raise NotFoundError(f"Model parameter '{key}' not found")
 
     def __setitem__(self, key, value: T.Any):
         return self.set(key, value)
@@ -232,11 +235,14 @@ class ParameterSet:
     def to_api_data(self) -> dict:
         return {name: param.to_api_data() for name, param in self._parameters.items()}
 
-    def get(self, name: str) -> T.Any:
+    def get_value(self, name: str) -> T.Any:
         return self._parameters[name].value
 
     def set(self, name: str, literal: T.Any):
-        self._parameters[name].expression = str(literal)
+        try:
+            self._parameters[name].expression = str(literal)
+        except KeyError:
+            raise NotFoundError(f"Model parameter '{name}' not found")
 
 
 class ModelParameters(ParameterSet):
@@ -250,6 +256,8 @@ class Model:
     """
 
     def __init__(self, data):
+        # TODO: for modifications, we'll need full representation of model.
+        # We also already fully load models whenever we access a block. Might as well do it here.
         self._data = data
         self._graph: ModelGraph = None
         self._datasource_data_by_uuid: T.Dict[str, pd.DataFrame] = {}  # block_uuid -> data (pd.DataFrame)
@@ -335,40 +343,60 @@ class Model:
         # if resp is not None and resp.get("version") is not None:
         #     self._data["version"] = resp["version"]
 
+    # todo traverse given path
+    def _find_block_by_path(self, path: str, autoload=False):
+        """
+        Find a block by path name
+        """
+        if not self.is_loaded:
+            if not autoload:
+                raise NotLoadedError(N(f"Model '{self}' is not loaded"))
+            self.load()
+        return self._graph.find_block_by_path(path)
+
+    # todo: do a search the model graph that constructs path 2
+    # todo: update so that one word argument is treated as a path.
+    # e.g. 'Adder_0' is a name by default. Must say find_block(path='Adder_0') to force path
     def find_block(
-        self, pattern: str = None, name: str = None, uuid: str = None, type: str = None, case=False, autoload=True
+        self, pattern: str = None, name: str = None, path: str = None, type: str = None, case=False, autoload=True
     ):
         """
-        Find a block by name.
+        Find a block by name or path.
         """
-        blocks = self.find_blocks(pattern=pattern, name=name, uuid=uuid, type=type, case=case, autoload=autoload)
+
+        if path:
+            return self._find_block_by_path(path, autoload=autoload)
+        elif is_path(pattern):
+            return self._find_block_by_path(pattern, autoload=autoload)
+
+        # possibly multiple
+        blocks = self.find_blocks(pattern=pattern, name=name, type=type, case=case, autoload=autoload)
         if len(blocks) == 0:
             raise NotFoundError(
                 N(
                     f"No block was found in model '{self}' "
-                    f"(pattern='{pattern}' name='{name}' uuid='{uuid}' type='{type}')"
+                    f"(pattern='{pattern}' name='{name}' path='{path}' type='{type}')"
                 )
             )
         if len(blocks) > 1:
             raise NotFoundError(
                 N(
                     f"Multiple matching blocks found in model '{self}' "
-                    f"(pattern='{pattern}' name='{name}' uuid='{uuid}' type='{type}'): {blocks}"
+                    f"(pattern='{pattern}' name='{name}' path='{path}' type='{type}'): {blocks}"
                 )
             )
         return blocks[0]
 
-    def find_blocks(
-        self, pattern: str = None, name: str = None, uuid: str = None, type: str = None, case=False, autoload=True
-    ):
+    # todo: do a search the model graph that constructs path 3
+    def find_blocks(self, pattern: str = None, name: str = None, type: str = None, case=False, autoload=True):
         """
-        Find all blocks that match a name.
+        Find all blocks that match
         """
         if not self.is_loaded:
             if not autoload:
                 raise NotLoadedError(N(f"Model '{self}' is not loaded"))
             self.load()
-        return self._graph.find_blocks(pattern=pattern, name=name, uuid=uuid, type=type, case=case)
+        return self._graph.find_blocks(pattern=pattern, name=name, type=type, case=case)
 
     def get_block_path(self, block, autoload=True):
         if not self.is_loaded:
@@ -386,10 +414,21 @@ class Model:
     #     """
     #     return run_simulation(self, parameters=parameters, wait=wait, no_sync=no_sync, ignore_cache=ignore_cache)
 
-    def set_parameters(self, parameters: dict):
+    def set_parameters(self, parameters: dict, save=False):
+        """Set model parameters.
+
+        :param parameters: A dictionary of the model parameters and their desired values.
+            Does not create new parameters.
+        :type parameters: dict
+        :param save: Option to save the parameter changes to the model, defaults to False
+        :type save: bool, optional
+        """
         paramset = ParameterSet(parameters)
         for k, v in paramset.items():
             self._parameters[k] = v
+        if save:
+            request_data = {"parameters": self.parameters.to_api_data()}
+            Api.model_parameter_update(self.uuid, request_data)
 
     def get_parameters(self) -> ModelParameters:
         return self._parameters
