@@ -21,6 +21,10 @@ class Project:
             return f"<{self.__class__.__name__} title='{self['title']}' uuid='{self['uuid']}'>"
         return f"<{self.__class__.__name__} title='{self['title']}'>"
 
+    @property
+    def uuid(self) -> str:
+        return self["uuid"]
+
 
 def get_project(title: str = None, case=False) -> Project:
     """
