@@ -15,7 +15,7 @@ from pycollimator.i18n import N
 from pycollimator.log import Log
 from pycollimator.models import Model, ModelParameters, load_model
 from pycollimator.simulation_hashed_file import SimulationHashedFile
-from pycollimator.utils import is_uuid
+from pycollimator.utils import is_path, is_uuid
 
 
 class LinearizationResult:
@@ -104,11 +104,12 @@ class SimulationResults:
     def __getitem__(self, item):
         if item == "time":
             return self._raw_df["time"]  # pylint: disable=unsubscriptable-object
-        if is_uuid(item):
-            blk = self._model.find_block(uuid=item)
-        elif isinstance(item, Block):
-            blk = self._model.find_block(uuid=item.uuid)
+        if isinstance(item, Block):
+            blk = item
+        elif is_path(item):
+            return self.to_pandas(path=item)
         else:
+            # todo: do a search the model graph that constructs path 1
             try:
                 blk = self._model.find_block(name=item, case=True)
             except NotFoundError:
@@ -116,50 +117,49 @@ class SimulationResults:
                     blk = self._model.find_block(name=item, case=False)
                 except NotFoundError:
                     blk = self._model.find_block(item)
-        df = self.to_pandas(uuid=blk.uuid)
+        df = self.to_pandas(path=blk.path)
         return df
 
-    def to_pandas(self, pattern=None, name=None, uuid=None, type=None, case=False) -> pd.DataFrame:
-        if is_uuid(pattern):
-            uuid = pattern
-            pattern = None
+    def to_pandas(self, pattern=None, name=None, path=None, type=None, case=False, item=None) -> pd.DataFrame:
+        if pattern is None and name is None and path is None and type is None and item is None:
+            # if nothing is specified, return all columns
+            cols = self._raw_df.columns[1:]  # remove time column and add as index later
+        else:  # find specific block(s) to return results of
+            # matches regardless of port specified or not, as well as all outputs of specified block
+            if is_path(pattern):
+                block_paths = [pattern]
+            elif path:
+                block_paths = [path]
+            elif isinstance(item, Block):
+                block_paths = [item.path]
+            else:  # when given name or type, must query for blocks first.
+                blocks = self._model.find_blocks(pattern=pattern, name=name, type=type, case=case)
+                block_paths = []
+                for block in blocks:
+                    block_paths.append(block.path)
+            cols = []
+            for col in self._raw_df.columns:
+                last_index = col.rfind(".")
+                block_path = col[: col.rfind(".")]
+                if (last_index != -1 and (block_path in block_paths)) or (col in block_paths):
+                    cols.append(col)
+        if len(cols) == 0:
+            raise NotFoundError(
+                N(
+                    f"Simulation results '{self}' of model '{self._model}' has no results for "
+                    f"(pattern='{pattern}' name='{name}' path='{path}' type='{type}' case='{case}' item='{item}') "
+                )
+            )
 
-        blocks = self._model.find_blocks(pattern=pattern, name=name, uuid=uuid, type=type, case=case)
-
-        cols_by_uuid = []
-        for block in blocks:
-            for col in self._raw_df.columns:  # type: str
-                if col.startswith(block.uuid):
-                    cols_by_uuid.append(col)
-
-        cols_by_path = []
-        for col in cols_by_uuid:
-            path = self._model.get_block_path(col.split("[")[0])
-            idx = col.split("[")[1].split("]")[0]
-            if path is not None:
-                if idx == "0":
-                    cols_by_path.append(path)
-                else:
-                    cols_by_path.append(f"{path}[{idx}]")
-
-        df = self._raw_df[cols_by_uuid]  # pylint: disable=unsubscriptable-object
-        df.columns = cols_by_path
+        df = self._raw_df[cols]  # pylint: disable=unsubscriptable-object
+        df.columns = cols
         df.index = self._raw_df["time"]  # pylint: disable=unsubscriptable-object
         df.index.name = "time"
         return df
 
     @property
     def columns(self) -> list:
-        pd_columns = self.to_pandas().columns.tolist()
-        cols = []
-        for col in pd_columns:
-            uuid = col.split("[")[0]
-            block = is_uuid(uuid) and self._model.find_block(uuid=uuid) or None
-            if block is not None:
-                cols.append(block.path)
-            else:
-                cols.append(col)
-        return cols
+        return self.to_pandas().columns.to_list()
 
 
 class SimulationLogLine:
@@ -212,6 +212,7 @@ class Simulation:
         return self._data["uuid"]
 
     @property
+    # this is last status, not latest as it doesn't request to backend.
     def status(self) -> str:
         return self._data["status"]
 
@@ -342,6 +343,7 @@ def _run_simulation_common(model, linearization, parameters, wait, no_sync, igno
 
     # Prepare DataSource files
     hashed_files = model._get_hashed_files()
+    # make special case for file name
     overrides = [
         {
             "expression": f'__hashed_file("{hashed_file.hash}", "{hashed_file.content_type}")',
